@@ -1,10 +1,10 @@
 # -*- perl -*-
 
 #
-# $Id: Kwalify.pm,v 1.2 2006/11/23 21:19:59 eserte Exp $
+# $Id: Kwalify.pm,v 1.3 2007/01/10 22:51:32 eserte Exp $
 # Author: Slaven Rezic
 #
-# Copyright (C) 2006 Slaven Rezic. All rights reserved.
+# Copyright (C) 2006,2007 Slaven Rezic. All rights reserved.
 # This package is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 #
@@ -16,7 +16,7 @@ package Schema::Kwalify;
 
 use strict;
 use vars qw($VERSION);
-$VERSION = sprintf("%d.%02d", q$Revision: 1.2 $ =~ /(\d+)\.(\d+)/);
+$VERSION = sprintf("%d.%02d", q$Revision: 1.3 $ =~ /(\d+)\.(\d+)/);
 
 use Kwalify qw();
 
@@ -26,7 +26,7 @@ sub new {
 
 sub validate {
     my($self, $schema, $data) = @_;
-    Kwalify::kwalify($schema, $data);
+    Kwalify::validate($schema, $data);
 }
 
 1;
